function read_global_header(path::String)
    data = open(path, "r")
    g_header = zeros(Int32, 4)
    read!(data, g_header)
    close(data)
    a = GHeader(g_header)
    return a
end

function read_CHeader(path::String)
    gh = read_global_header(path)
    
    data = open(path, "r")
    seek(data, gh.hsize)
    
    aux_f = zeros(Float64, 6)
    aux_i = zeros(Int32, 4)
    theta = zeros(Float64, 6)
    a = Vector{CHeader}(undef, gh.ncorr)
    for k = 1:gh.ncorr
        read!(data, aux_f)
        read!(data, theta)

        qs1 = read(data, Int32)
        if qs1 != 0
            qn1 = read(data, Int32)
            qeps1 = read(data, Float64)
            q1 = Sm(qs1, qn1, qeps1, 1)
        else
            q1 = Sm(qs1, 1)
        end

        qs2 = read(data, Int32)
        if qs2 != 0
            qn2 = read(data, Int32)
            qeps2 = read(data, Float64)
            q2 = Sm(qs2, qn2, qeps2, 1)
        else
            q2 = Sm(qs2, 1)
        end


        gs1 = read(data, Int32)
        if gs1 != 0 && gs1 != 3 && gs1 != 4
            gn1 = read(data, Int32)
            geps1 = read(data, Float64)
            g1 = Sm(gs1, gn1, geps1, 2)
        elseif gs1 == 3 || gs1 == 4
            g1 = Sm(gs1, q1.niter, q1.eps, 2)
        else
            g1 = Sm(gs1, 2)
        end
        
        gs2 = read(data, Int32)
        if gs2 != 0 && gs2 != 3 && gs2 != 4
            gn2 = read(data, Int32)
            geps2 = read(data, Float64)
            g2 = Sm(gs2, gn2, geps2, 2)
        elseif gs1 == 3 || gs1 == 4
            g2 = Sm(gs2, q2.niter, q2.eps, 2)
        else
            g2 = Sm(gs2, 2)
        end
        
        read!(data, aux_i)
        a[k] = CHeader(aux_f, aux_i, theta, [q1, q2, g1, g2])
    end
    close(data)
    return a
end


function read_mesons(path::String, g1::Union{String, Nothing}=nothing, g2::Union{String, Nothing}=nothing; id::Union{Int, Nothing}=nothing)
    
    isnothing(g1) ? t1=nothing : t1 = findfirst(x-> x==g1, gamma_name) - 1
    isnothing(g2) ? t2=nothing : t2 = findfirst(x-> x==g2, gamma_name) - 1
    if isnothing(id)
        bname = basename(path)
        m = findfirst(r"[A-Z][0-9]{3}r[0-9]{3}", bname)
        id = parse(Int64, bname[m[2:4]])
    end

    data = open(path, "r")
    g_header = read_global_header(path)
    c_header = read_CHeader(path)

    ncorr = g_header.ncorr
    tvals = g_header.tvals
    nnoise = g_header.nnoise

    fsize = filesize(path)

    datsize = 4 + sum(c.dsize for c in c_header)*tvals*nnoise #datasize / ncnfg
    ncfg =  Int32((fsize-g_header.hsize-sum(c.hsize for c in c_header)) / datsize)

    corr_match = findall(x-> (x.type1==t1 || isnothing(t1)) && (x.type2==t2 || isnothing(t2)), c_header)

        
    seek(data, g_header.hsize + sum(c.hsize for c in c_header))

    res = Array{CData}(undef, length(corr_match))

    data_re = Array{Float64}(undef, length(corr_match), ncfg, tvals)
    data_im = zeros(length(corr_match), ncfg, tvals)
    vcfg = Array{Int32}(undef, ncfg)

    for icfg = 1:ncfg
        vcfg[icfg] = read(data, Int32)
        c=1
        for k = 1:ncorr
            if k in corr_match 
                if c_header[k].is_real == 1
                    tmp = Array{Float64}(undef, tvals*nnoise)
                    read!(data, tmp)
                    tmp2 = reshape(tmp, (nnoise, tvals))
                    tmp2 = mean(tmp2, dims=1)
                    data_re[c, icfg, :] = tmp2[1, :]
                elseif c_header[k].is_real == 0
                    tmp = Array{Float64}(undef, 2*tvals*nnoise)
                    read!(data, tmp)
                    tmp2 = reshape(tmp, (2, nnoise, tvals))
                    tmp2 = mean(tmp2, dims=2)
                    data_re[c, icfg, :] = tmp2[1, 1, :]
                    data_im[c, icfg, :] = tmp2[2, 1, :]

                end
                c += 1
            else
                seek(data, position(data)  + c_header[k].dsize*tvals*nnoise)
            end
        

        end
    end

    for c in 1:length(corr_match)
        res[c] = CData(c_header[corr_match[c]], vcfg, data_re[c, :, :], data_im[c, :, :], id)
    end
    close(data)

    return res
end


function read_ms1(path::String, v::String="1.2")
    data = open(path, "r")
    nrw = read(data, Int32)

    if v == "1.4" || v =="1.6"
        nfct = Array{Int32}(undef, nrw)
        read!(data, nfct)
        nfct_inheader = 1
    elseif v=="1.2"
        nfct = ones(Int32, nrw)
        nfct_inheader = 0
    else
        println("Error: Version not supported")
        return nothing  
    end
    nsrc = Array{Int32}(undef, nrw)
    read!(data, nsrc)
    glob_head_size = 4 + 4*nrw*(nfct_inheader + 1)
    datsize = 4 + 2*8*sum(nsrc .* nfct)

    fsize = filesize(path)

    ncnfg = Int32((fsize - glob_head_size)/datsize)
    r_data = Array{Array{Float64}}(undef, nrw)
    
    [r_data[k] = zeros(Float64, nfct[k], nsrc[k], ncnfg) for k = 1:nrw]
    vcfg = Array{Int32}(undef, ncnfg)
    for icfg = 1:ncnfg
        vcfg[icfg] = read(data, Int32)
        for irw = 1:nrw
            for ifct = 1:nfct[irw]
                tmp = zeros(Float64, nsrc[irw])
                seek(data, position(data) + 8 * nsrc[irw])
                read!(data, tmp)
                r_data[irw][ifct, :, icfg] = tmp[:]
            end
        end
    end
    
    W = zeros(Float64, nrw, ncnfg)
    [W[k, :] = prod(mean(exp.(.-r_data[k]), dims=2), dims=1) for k = 1:nrw]
    close(data)
    return W
end